;-----------------------------------------------------------------------------
;* Musterprogramm fuer AVR-Assembler (avra)       letzte Aenderung: 23.8.2012
;* Version: 0.0
;* Prozessor: Atmega8
;* Schaltschema: digio.png, Port B0,B1,B2 mit LEDs verbunden
;*                          Port D2,D3 mit Schaltern verbunden
;* Autor: Rolf Pfister
;* Copyright: Freeware
;* History:
;* 4.3.2009	Erstellung
;* 23.8.2012	Comments in english, new version of mainloop2
;*
;-----------------------------------------------------------------------------
	.include "m8def.inc" 	; Definitionsdatei fuer den Prozessortyp
; Reset und Interruptvectoren	;VNr. Beschreibung
begin:	rjmp	main		; 1   Power On Reset
	reti			; 2   Int0-Interrupt
	reti			; 3   Int1-Interrupt
	reti			; 4   TC2 Compare Match
	reti			; 5   TC2 Overflow
	reti			; 6   TC1 Capture
	reti			; 7   TC1 Compare Match A
	reti			; 8   TC1 Compare Match B
	reti			; 9   TC1 Overflow
	reti			; 10  TC0 Overflow
	reti			; 11  SPI, STC = Serial Transfer Complete
	reti			; 12  UART Rx Complete
	reti			; 13  UART Data Register Empty
	reti			; 14  UART Tx Complete
	reti			; 15  ADC Conversion Complete
	reti			; 16  EEPROM ready
	reti			; 17  Analog Comperator
	reti			; 18  TWI (I2C) Serial Interface
	reti			; 19  Store Program Memory ready
;-----------------------------------------------------------------------------
; Start, Power On, Reset
main:	ldi r16, RAMEND&0xFF
        out SPL, r16  	     ; Init Stackpointer L
	ldi r16, (RAMEND>>8)
        out SPH, r16  	     ; Init Stackpointer H

;; Init-Code
	ldi r16, 0x07		;PB0,PB1,PB2 as outputs (to the 3 LEDs)
        out DDRB, r16		;Data Direction Register B
	ldi r16, 0
	out PORTB, r16		;all LEDs off
        ldi r17, 0x00
        out DDRD, r17		;All lines Port D as inputs
        ldi r17, 0b00001100	;D2,D3 pullup resistors
        out PORTD, r17	        ;PortD activate pullup resistors

;	rjmp mainloop2		;uncomment to try out second version
	
;-----------------------------------------------------------------------------
; first version from 2009
mainloop:
	in r17, PIND		;read Port D
	andi r17, 0x04		;check PD2
	brne label1		;branch if not equal
	ori r16, 0x01		;red LED on
	rjmp label2
label1: andi r16, 0xFE		;red LED off
label2:	in r17, PIND
	andi r17, 0x08		;check PD3
	brne label3
	ori r16, 0x04		;green LED on
	rjmp label4
label3:	andi r16, 0xFB		;green LED off
label4:
	out PORTB, r16
	rjmp mainloop

;-----------------------------------------------------------------------------
; second version (2012) making the same thing with shorter code:
mainloop2:
	ldi r16, 0		;LEDs off when no buttons pressed
	sbis PIND, 2		;check bit 2 on Port-Input-D (skip next instruction if bit set)
	ori r16, 0x01		;red LED on
	sbis PIND, 3		;check bit 3 on Port-Input-D
	ori r16, 0x04		;green LED on
	out PORTB, r16
	rjmp mainloop2
